#include "StdAfx.h"
#include "simplegenerator.h"
#include "Core\HField.h"

Device *SimpleGeneratorMaker() { return new SimpleGenerator; };
void SimpleGeneratorKiller(Device *spr) { delete spr; };

SimpleGenerator::SimpleGenerator(void)
{
	lifeptrs.maker = SimpleGeneratorMaker;
	lifeptrs.killer = SimpleGeneratorKiller;
	strncpy(lifeptrs.nametag, "EXSG", 4);
	SetLinks(0, 1);	// generators have 0 inputs by default. we'll just re-emphasize that..

// Let's add some parameters!
	AddParam(Parameter("Roughness", 0.1f, 0.0f, 1.0f));
// An integer parameter. We'll then turn it into an enum (selection)
	Parameter p("Type", 0, 0, 1);	// integer with default of 0, min of 0, max of 1
	p.SetupEnum();	// convert to enum type, setup help information areas.
	p.setEnumString(0, "Noise");	// Value 0 == simple
	p.setEnumString(1, "Blank");	// Value 1 == Blank
// finally, add it
	AddParam(p);

// We can add help strings to our parameters after the fact, or during creation. here's after the fact:
	params.GetParam(0)->setHelpString("Roughness controls the amount of noise added to the terrain");
	params.GetParam(1)->setHelpString("Choose from Simple Noise or a Blank Map!"); 

}

#define SG_ROUGH			ParmFRef("Roughness")
#define SG_TYPE				ParmIRef("Type")


SimpleGenerator::~SimpleGenerator(void)
{

}

// You are REQUIRED to write out and read in your 4-byte Device ID tag, as shown below. It is also recommended that your device
// have an internal version number that you keep track of to allow backwards-compatibility if you ever change your Device save layout...
// Also, make sure you call your super class's Load and Save functions as shown here. Failure to do so will cause major problems.
bool SimpleGenerator::Load(std::istream &in, LoadContext &context) {
	char tag[5];
	in.read( tag, 4);
	if (strncmp(tag, lifeptrs.nametag, 4) == 0) {
		int ver = 0;
		in.read((char*) &ver, 1);
		return Generator::Load(in, context);
	}
	else
		return false;
};

bool SimpleGenerator::Save(std::ostream &out, SaveContext &context) {
	out.write( lifeptrs.nametag, 4);
	int ver = 1;
	out.write((char*) &ver, 1);
	return Generator::Save(out, context);
};

bool SimpleGenerator::Activate(BuildContext &context) {
// retrieve parameter values;
	float roughness = SG_ROUGH;
	int type = SG_TYPE;

// Create a new heightfield using current specified world size
	HFPointer map = GetNewHF(context);

	srand(12121);
	// Now fill the map with data depending on the type of noise the user wants

	if (type == 0) {	// fill map with simple noise
		for (int y = 0; y < map->h(); y++) {
			context.ReportDeviceProgress(this, y, map->h());  
			for (int x = 0; x < map->w(); x++) {
				float value = roughness * float(rand() % 65535) / 65535.0f;
				(*map)[Coord(x,y)] = value;
			}
		}
	}
	else  {	// blank map!
		map->Clear();
	}

	map->ClampRange();	// Ensures that all data is in the range 0..1. Just in case.

// Handoff to output(s)
	StoreData(map,0, context); // pass the heightfield to our output stage.
// Success!
	return true;
}
