#pragma once

#include "WMPacket.h"

/** Abstract WMPacket-derived base class that coverts some "area" of space
*/
class WMAreaPacket : public WMPacket  {
public:
	virtual WMAreaPacket &operator=(const WMAreaPacket &rhs);

	virtual void Init(const PacketContext &_size);
	virtual bool queryType(const int type);
	virtual bool Load( std::istream &input ) override;
	virtual bool Save( std::ostream &output ) override;

	/// @name Area/Size functions
	//@{
	/// Retrieve context reference for modifications
	PacketContext &getContext() { return areacontext; };
	/// Return const PacketContext
	const PacketContext &getContext() const { return areacontext; };
	/// width of heightfield
	inline int w() const { return areacontext.w(); };
	/// height of heightfield
	inline int h() const { return areacontext.h(); };
	/// area of heightfield (w*h)
	inline size_t area() const { return areacontext.resarea(); };
	/// size returned as a Coord (x,y) pair
	inline Coord getsize() const { return areacontext.getRes(); };
	/// *worldsize* returns as a floating point pair
	inline CoordF getRegionSize() const { return areacontext.getRegionSize(); };
	/// Return the worldspace region of this packet
	inline Rect4F	getRegion() const { return areacontext.getRenderExtent4F(); };

	PacketContext::SPACETYPE getSpatialType() { return areacontext.getSpatialType(); };

	inline float getDScale() const { return areacontext.getDScale(); };
	inline float getVScale() const {  return areacontext.getVscale(); };
	//@}
protected:
	WMAreaPacket();
	virtual ~WMAreaPacket();
	friend class WMPacketFactory;

	PacketContext areacontext;
};

inline AreaPointer AREAPACKET(PacketPointer pack) { 
	if (pack) {
		bool ok = pack->queryType(WMP_areabase);
		if (!ok)
			return NULL;
	}
	return AreaPointer(static_cast<WMAreaPacket*>(pack.get()) ); 
};
