#pragma once
/** @file BasicDevices.h
	Class definitions for Parameter-based devices

	Additional classes defined in Filter.h
*/
#include "Device.h"
#include "PluginHeader.h"

/** Base class for Parameter-based devices.
	@ingroup devices

	Parameter devices are used to manipulate values (typically scalar or integer, but can be otherwise)
	to be fed to other devices in the network. These are particularily invaluable for macros.
*/
class ParamDevice : public Device {
public:
	ParamDevice();
	virtual dword Color() { return 0x0f4dddd; };
	virtual int GetDeviceFamily() { return TYPE_PARAM; };

	virtual bool isViewable() { return false; };	///< Parameter Devices do not in of themselves produce a viewable output
	virtual bool isMaskable() { return false; };	///< Parameters cannot be masked

	virtual int getSpeedClass() { return 5; };


	virtual char *GetInputName(int slot);
	virtual char *GetOutputName(int slot);

	virtual PacketPointer	getDefaultOutput(PSTATE preview) override;
	virtual Device *		getDisplayDevice() override;

	virtual bool isOverrideCapable();

};

/// Scalar-based device family. These devices are extremely simple and simply produce or consume scalar (0..1) values.
/// @ingroup devices
class ScalarDevice : public ParamDevice {
public:
	ScalarDevice();

	virtual char *GetInputName(int slot);
	virtual char *GetOutputName(int slot);
};

/// Coordinate devices
/// Coordinate devices produce or consume coordinate informatoin. This is currently extremely basic
/// @ingroup devices
class CoordDevice : public ParamDevice {
public:
	CoordDevice();

	virtual char *GetInputName(int slot);
	virtual char *GetOutputName(int slot);
};
