#pragma once

#include "Context.h"
#include "Rescaler.h"

#include <iostream>

namespace WorldMachine {
	class ContextOverride: public PacketContext {
	public:
		ContextOverride();

		bool operator==(ContextOverride const&rhs) const;
		bool operator!=(ContextOverride const&rhs) const { return !operator==(rhs); };

		/// Replaces our contents with the contents of the other, IF and only if the others' contents are enabled.
		/// This helps us compose an override from multiple sources
		bool replaceFrom(ContextOverride const&other);

		enum SCALE_METHOD { SCALED, FIXED };

		bool Load(std::istream &in);
		bool Save(std::ostream &out);

		/// Returns true if overriding anything. Query further to find out what...
		bool isActive() const { return isResolutionEnabled() || isSpatialEnabled(); };
		/// Overriding resolution
		bool isResolutionEnabled() const { return enabled; };
		/// Overriding spatial type
		bool isSpatialEnabled() const { return this->getSpatialType() != PacketContext::PARENTSPACE; };

		void setResolutionEnabled(bool _e) { enabled = _e; };

		SCALE_METHOD getScaleMethod() const { return method; };
		void setScaleMethod(SCALE_METHOD s) { method = s;};
		bool isFixedRes() const { return method == FIXED; };

		Rescale::RescaleInfo &getRescaleInfo() { return rescale_info; };
		void setRescaleInfo(Rescale::RescaleInfo t) { rescale_info = t; };


		void setScalar(float sc) { scalar = sc; };
		float getScalar() const { return scalar; };

		// Modify the destination context if we are overriding it.
		bool overrideResolutionContext( BuildContext &destcontext ) const;
		bool overrideSpatialContext( BuildContext &destcontext ) const;
		/// Return what the given resolution would be if it was modified by the context override. This means either will return a local or scaled result
		int getOverrideResolution(int in_res) const;

	private:

		/// Private version that also considers if this is a tiled build or not. This version is used when actually overriding a context.
		int getOverrideResolution(BuildContext &destcontext) const;
		SCALE_METHOD method;
		float scalar;
		Rescale::RescaleInfo rescale_info;
		bool enabled;
	};
}

