#pragma once

#include "Device.h"
#include "BuildResultSet.h"

/** @file A set of operation to query the combination of a Device and its build status.
*/

namespace WorldMachine {

	/** A Query object that combines the information of a Device and its results for querying state.
	This is necessary because we split the Device class to no longer hold its mutable contents; instead that kind of per-build data is stored in the BuildResultSet class.
	Thus, the queries we want require information from both, and fit in neither; instead we encapsulate them here.

	The Query will throw an exception on construction if either the device or its results don't exist.

	Lifetime: a query must not outlive its associated device and deviceresults.
	*/
	class DeviceQuery {
	public:
		// construct query object directly from a device and its results
		DeviceQuery(Device *dev, const DeviceResults &res);
		// construct query object indirectly from a device; the build result will be queried from the preview build results of its attached world.
		DeviceQuery(Device *dev, PSTATE state = P_PREVIEW);

		/// Is the device in localspace?
		bool isLocalSpace();

		/// Is this device in localspace because of its inputs?
		bool isLocalSpaceThroughInputs();

		// Is the default output of this default in localspace?
		bool isDefaultOutputLocalspace();

		/// Retrieve the build packetcontext for this device
		PacketContext getContext();
	private:
		Device const*device;
		DeviceResults const*results;
	};

}

