#pragma once

#include "datatype_generics.h"
#include "WMException.h"

template < class T > 
template <typename visitor_type >
void WMGridPacket<T>::visitAll( visitor_type visitor )
{
	T* itr = datagrid;
	size_t index = 0;
	RectF area = areacontext.getRenderExtent();
	Vector2D scaling(area.width / double(areacontext.w()-1), area.height / double(areacontext.h()-1));
	for (int y = 0; y < areacontext.h(); y++) {
		CoordF wloc(area.x, area.y + float(scaling.y * double(y)) );
		for (int x = 0; x < areacontext.w(); x++) {
			wloc.x = float(area.x + float(scaling.x * double(x)));
			GridLocation loc(*itr, Coord(x,y), wloc, index);
			visitor(loc);
			++itr;
			++index;
//			wloc.x += scaling.x;
		}
	}
}


template<class T> WMGridPacket<T>::WMGridPacket() {
	datagrid = NULL;
};

template<class T> WMGridPacket<T>::~WMGridPacket() {
	WMFree(datagrid);
	datagrid = NULL;
}

template<class T> WMGridPacket<T> &WMGridPacket<T>::operator=(const WMGridPacket<T> &other) {
	if (this != &other) {
		WMAreaPacket::operator=(other);
		InitFrom(other);	// make our own data
		if (other.datagrid) {
			memcpy(datagrid, other.datagrid, area()*sizeof(T));
		}
	}
	return *this;
}

/*
template <class T> bool WMGridPacket<T>::queryType(const int type) {
	if (type == WMP_gridbase) {
		return true;
	}
	else
		return WMPacket::queryType(type);
};
*/

template<class T> void WMGridPacket<T>::Init(const PacketContext &_size) {
	resetContents(); // clear previous
	areacontext = _size;
	size_t allocsize = sizeof(T)*areacontext.resarea();
	if (0 == allocsize)
		return;

	datagrid = (T*) WorldMachine::core->memory->Alloc(allocsize, true);
}

template<class T> void WMGridPacket<T>::InitFrom(const WMGridPacket<T> &other) {
	Init(other.areacontext);
};

template<class T> void WMGridPacket<T>::ClearTo(T clearval) {
	assert(datagrid);
	size_t area = areacontext.resarea();
	for (size_t i = 0; i < area; ++i) {
		datagrid[i] = clearval;
	}
}

template<class T> bool WMGridPacket<T>::Load( std::istream &input )
{
	if (!WMAreaPacket::Load(input))
		return false;
	int ver = 0;
	read(input, ver, 1);
	if (ver != 1)
		return false;

	Init(areacontext);
	unsigned long long size = 0;
	read(input, size, 8);
	assert(area() * sizeof(T) == size);

	if (size > (std::numeric_limits<size_t>::max)()) { // cannot load properly
		throw WorldMachine::ComponentIOException("Tried to load packet larger than maximum addressible memory!");
	}
	read(input, *datagrid, size_t(size));

	return true;
}

template<class T> bool WMGridPacket<T>::Save( std::ostream &output )
{
	WMAreaPacket::Save(output);

	int ver = 1;
	write(output, ver, 1);

	unsigned long long size = area() * sizeof(T);
	output.write((char*) &size, 8);
	output.write((char*) datagrid, size);
	return true;
}


template<class T> void WMGridPacket<T>::resetContents() {
	WMFree(datagrid);
	datagrid = NULL;
};

// do a simple index lookup
template<class T> size_t WMGridPacket<T>::getIndex(const Coord &loc) {
	return (size_t(loc.y*w())) + size_t(loc.x);
}

// return the stride distance from one line to the next, in array elements
template<class T> size_t WMGridPacket<T>::getStride() {
	return (size_t) w();
}

// The inline data accessors
template<class T> inline T WMGridPacket<T>::value(const Coord loc) const {
	return datagrid[(size_t(loc.y)*size_t(w()) ) + size_t(loc.x)];
};


template <class T>
T const& WMGridPacket<T>::operator[]( const size_t index ) const
{
	return datagrid[index];
}

template<class T> inline T &WMGridPacket<T>::operator[](const size_t index) {
	return datagrid[index];
}

template<class T> inline T &WMGridPacket<T>::operator[](const Coord loc) {
	return datagrid[(size_t(loc.y)*size_t(w()) ) + size_t(loc.x)];
}

template<class T> inline T &WMGridPacket<T>::operator[](const Vector2 loc) {
	return datagrid[size_t(ffi(loc.y))*size_t(w()) + size_t(ffi(loc.x))];
}

template<class T> inline T *WMGridPacket<T>::GetDataPtr() {
	return datagrid;
}

template <class T>
T WMGridPacket<T>::valueUV( const CoordF uv ) const
{
	// convert UV to full coordinates
	CoordF loc = uv * i2f(areacontext.getRes() - Coord(1,1));
	assert(loc.x >= 0 && loc.x < areacontext.getRes().x);
	assert(loc.y >= 0 && loc.y < areacontext.getRes().y);

	return valueF(loc);
}


template<class T> inline T WMGridPacket<T>::valueSafe(const Coord loc) const {
	Coord cliploc = loc;
	if (cliploc.x < 0)
		cliploc.x = 0;
	else if (cliploc.x >= w())
		cliploc.x = w()-1;
	if (cliploc.y < 0)
		cliploc.y = 0;
	if (cliploc.y >= h())
		cliploc.y = h()-1;
	return datagrid[(size_t(cliploc.y)*size_t(w()) ) + size_t(cliploc.x)];
};

template<class T> inline T WMGridPacket<T>::valueSafeZero(const Coord loc) const {
	Coord cliploc = loc;
	if (cliploc.x < 0)
		return 0;
	else if (cliploc.x >= w())
		return 0;
	if (cliploc.y < 0)
		return 0;
	if (cliploc.y >= h())
		return 0;
	
	return datagrid[(size_t(cliploc.y)*size_t(w()) ) + size_t(cliploc.x)];
};


template<class T> inline T WMGridPacket<T>::valueSafeF(const CoordF loc) const {
	Coord a;
	a.x = (int)loc.x;
	a.y = (int)loc.y;
	Coord b = a, c = a, d = a;
	b.x++;
	c.y++;
	d.x++;
	d.y++;

	float u = loc.x - a.x;
	float v = loc.y - a.y;

	T v1,v2,v3,v4;
	v1 = valueSafe(a);
	v2 = valueSafe(b);
	v3 = valueSafe(c);
	v4 = valueSafe(d);

	T i1 = lerp<T>(v1, v3,v);
	T i2 = lerp<T>(v2, v4,v);
	return lerp<T>(i1, i2,u);
};

template<class T> inline T WMGridPacket<T>::valueF(const CoordF loc) const {
	Coord a;
	a.x = (int)loc.x;
	a.y = (int)loc.y;
	Coord b = a, c = a, d = a;
	b.x++;
	c.y++;
	d.x++;
	d.y++;

	float u = loc.x - a.x;
	float v = loc.y - a.y;

	T v1,v2,v3,v4;
	v1 = value(a);
	v2 = valueSafe(b);
	v3 = valueSafe(c);
	v4 = valueSafe(d);

	T i1 = lerp<T>(v1, v3,v);
	T i2 = lerp<T>(v2, v4,v);
	return lerp<T>(i1, i2,u);
};

template<class T> void WMGridPacket<T>::determineRange(T &min, T &max) const {
	if (!datagrid)
		return;
// get largest, smallest size
	size_t size = area();

	assert(size > 0);
	T minval = datagrid[0], maxval = datagrid[0];
	for (size_t i = 0; i < size; i++) {
		T &value = datagrid[i];
		minval = WorldMachine::datatype::minval(value, minval);
		maxval = WorldMachine::datatype::maxval(value, maxval);
	}

	min = minval;
	max = maxval;
};
/*
template< class T> void WMGridPacket<T>::ClampRange() {
	T min, max;
	determineRange(min, max);
	size_t size = area();
	for (size_t i = 0; i < size; i++) {
		T &value = datagrid[i];
		value 
	}
}
*/
