#pragma once

class HField;
class WMPacket;
class BMPacket;

#include <string>
#include "common.h"
#include "HField.h"

enum FI_FORMAT { FI_NONE, FI_TERRAGEN, FI_RAW, FI_RAW16, FI_RAW32, FI_BT, FI_BMP, FI_TGA, FI_TIFF, FI_PNG, FI_HFZ};

// Define this if you want to compile HFZ support. HFZ is under lGPL license and so must be re-linkable, which is
// often unacceptable in 3rd party software. Load or save calls to that format will silently fail if this is not defined.
#define _USE_HFZ

/// Heightfield file export helper class
///	@ingroup data
class HFieldExporter
{
public:
	HFieldExporter();
	HFieldExporter(HFPointer hf);
	~HFieldExporter(void);

	void setMetadata(const char *txt) { metadata = txt; };
	void setRGB(bool _rgb) { rgb = _rgb; };

// RGB or HF Exporters
	bool SavePNG(const char *fname, int bitdepth = 16);

//// HF only exporters
// 8bit
	bool SaveTGA(const char *fname, bool colorize = false);	// Save as a 24bit TGA file
	bool SaveRAW(const char *fname);	// Save as a raw, 8bit int dump
	bool SaveBMP(const char *fname, bool colorize = false);
// 16bit+
	bool SaveRAW16(const char *fname);	// Save as a raw 16bit int dump
	bool SaveRAW32(const char *fname);	// Save as a raw, 32bit FP dump
	bool SaveTGA16(const char *fname);
	bool SaveEXR(const char *fname);
	bool SaveTIFF16(const char *fname);
	bool SavePGM(const char *fname);
// scale-carrying formats
	bool SaveBT(const char *fname);
	bool SaveTer(const char *fname, bool addpadding = true);	// Save in Terragen format
	bool SaveLev(const char *fname);
	bool SaveHFZ(const char *fname);	// HF2/HFZ format

private:
	// export data
	bool hasMetadata() {return !metadata.empty(); };
	bool rgb;
	std::string metadata;

	// export data
	HFPointer hf;
};

/// Heighfield (and BMPacket) file import helper class
/// @ingroup data
class HFieldImporter
{
public:
	HFieldImporter() { has_scale = false; };
	~HFieldImporter() { };

	/// Import Functionality
	/// set to true if you wish to produce BMPackets rather than HFields for those importers that support both
	void setRGB(bool _rgb) { rgb = _rgb; };

	bool hasMetadata() { return !metadata.empty(); };
	std::string &getMetadata() { return metadata; };

	bool hasScaleData() { return has_scale; };
	Vector2 getScaleData() { return scalerange; };

	bool hasAlpha();
	HFPointer getAlphaMask();

/// Master load function, will determine filetype from extension.
// can use this in place of the below if you want
	PacketPointer LoadFile(const char *fname);

// Bitmap to HF Converter (luminance convert)
	HFPointer	 ConvertToHF(BMPointer src);
// HF to Bitmap Converter (greyscale)
	BMPointer ConvertToBMP(HFPointer	 src);

// RGB or HField Importers
	PacketPointer LoadBMP(const char *fname);
	PacketPointer LoadTGA(const char *fname);
//	PacketPointer LoadJPG(const char *fname);
	PacketPointer LoadPNG(const char *fname);
	PacketPointer LoadTIFF(const char *fname);

// HF only importers
	HFPointer	 LoadTer(const char *fname);
	HFPointer	 LoadBT(const char *fname);
	HFPointer	 LoadHFZ(const char *fname);

	HFPointer	 LoadRAW(const char *fname);
	HFPointer	 LoadR16(const char *fname);
	HFPointer	 LoadR32(const char *fname);

	// Return the edge size for a RAW file or 0 if invalid
	bool VerifyRAWSize( const char * fname, const size_t bytes_per_pixel, size_t &out_fsize, int &out_edgesize );
private:
	bool AddScaleMetadata();
	bool AddPRJMetadata( std::ifstream & prj );
	// import data
	bool rgb;
	std::string metadata;

	HFPointer alpha;

	bool has_scale;
	CoordF scalerange;
};


FI_FORMAT GetFileType(const char *name);
void FlipPacket(PacketPointer wmp);