#pragma once
#include "WMPacket.h"
#include "Parameter.h"

/// A Parameter packet is simply a Parameter wrapped in a WMPacket for transfer through the WM device network.
///	@ingroup data
class ParamPacket : public WMPacket
{
public:
	static ParamPointer CreateNew();

	virtual PacketPointer Clone() const;
	ParamPacket &operator=(const ParamPacket &other);

	virtual size_t getHeapUsage() { return 0;	};

	static int getStaticType() { return WMP_parameter; };
	virtual int getType() override { return getStaticType(); };
	virtual char *getTypeName() { return "Parameter"; };
	virtual bool queryType(const int type);	

	virtual bool Load(std::istream &input);
	virtual bool Save(std::ostream &output);	

	virtual dword Color() { return 0x0ff0000; };

	int paramType() { return param.type; };
	VAL &paramVal() { return param.value; };	// reference to parameter value
	virtual void Clear();

	Parameter &getParam();
	void setParam(const Parameter &p);

protected:
	friend class WMPacketFactory;
	ParamPacket(void);
	~ParamPacket(void);
	virtual void resetContents() { };

	Parameter param;			// the included parameter
};

/// Wraps a single floating point value for transfer through the network.
/// Thus, it is perhaps the simplest WMPacket to be implemented.
/// @ingroup data
class ScalarPacket : public WMPacket 
{
public:
	static ScalarPointer CreateNew();
	ScalarPacket &operator=(const ScalarPacket &other);

	virtual size_t getHeapUsage() { return 0;	};

	virtual int getType() { return WMP_scalar; };
	virtual char *getTypeName() { return "Scalar"; };
	virtual void Clear() { };
	virtual PacketPointer Clone() const;

	virtual bool Load(std::istream &input);
	virtual bool Save(std::ostream &output);	

	virtual dword Color() { return 0x0770000; };

	float val() { return scalar; };
	void setval(float _newval) { scalar = _newval; };

protected:
	friend class WMPacketFactory;
	ScalarPacket(void);
	~ScalarPacket(void);
	virtual void resetContents() { };

	float scalar;
};

/** Accessor inline function to provide a safer way to cast WMPacket to ParamPacket
// in debug mode it will assert if the packet is not a ParamPacket; in release you pay nothing
// framework should never allow this anyways, but it never hurts to be careful!
	@relates ParamPacket
*/
inline ParamPointer PARAM(PacketPointer pack) { 
	if (pack) {
#ifdef _DEBUG
		bool ok = pack->getType() == WMP_parameter;
		if (!ok)
			return NULL;
#endif
	}
	return ParamPointer((ParamPacket*) pack.get()); 
};

/** Accessor inline function to provide a safer way to cast WMPacket to ScalarPacket
// in debug mode it will assert if the packet is not a ScalarPacket; in release you pay nothing
// framework should never allow this anyways, but it never hurts to be careful!
	@relates ScalarPacket
*/
inline ScalarPointer SCALAR(PacketPointer pack) { 
	if (pack) {
#ifdef _DEBUG
		bool ok = pack->getType() == WMP_scalar;
		if (!ok)
			return NULL;
#endif
	}
	return ScalarPointer((ScalarPacket*) pack.get()); 
};
