#pragma once
#include "wmpacket.h"

#include <string>

class WMPacket;

/** Encapsulation of text data for transfer through the device network.
	@ingroup data

*/
class TextPacket :
	public WMPacket
{
public:
	virtual TextPacket &operator=(const TextPacket &rhs);
	virtual PacketPointer Clone() const;			// Return a copy of this WMPacket-derived packet
	virtual size_t getHeapUsage() { return 0; };

	virtual bool Load(std::istream &input);
	virtual bool Save(std::ostream &output);

	static int getStaticType() { return WMP_text; };
	virtual int getType() override { return getStaticType(); };
	virtual char *getTypeName() { return "Text"; };
	virtual bool queryType(const int type);	

	virtual dword Color() { return 0x007f7f7f; };

	virtual void Clear();
/// Set the text to encapsulate with this packet.
	void setText(const char *txt);
/// Retrieve the payload text
	const char *getText() const;
protected:
	friend class WMPacketFactory;
	TextPacket(void);
	~TextPacket(void);
	virtual void resetContents() { };

	char *body;
};

/** Packet Cast for TextPacket
 Accessor inline function to provide a safe way to cast WMPacket to TextPacket.
 in debug mode it will assert if the packet is not a TextPacket

 @relates TextPacket

 */
inline TextPointer TPACKET(PacketPointer pack) { 
	if (pack) {
		bool ok = pack->getType() == WMP_text;
		if (!ok)
			return NULL;
	}
	return TextPointer((TextPacket*) pack.get()); 
};

/// Get a new Text Packet
/// @relates TextPacket
inline TextPointer GetNewTextPacket() {
	TextPointer tp = TPACKET(WorldMachine::core->memory->GetPacket(WMP_text));
	return tp; 
};